==> currents-api-types-0.0.29.tgz: package/auth/user/parsers.ts <==
import { z } from 'zod';
import { AUTH_USERS_API_GET_LIMIT } from './constants';
import { isUserId } from './id.utils';
import { AuthUsersResult, UserId } from '../../user/types';
import { auth } from 'firebase-admin';
import { makeLimitParser, queryLimitParser } from '../../common/parsers/queryParams';

// for assertions like one type *looks like* a subtype of another
// eslint-disable-next-line @typescript-eslint/no-unused-vars
export type Subset<T extends U, U> = U;

// for 'get' query params, .or shenanigans
export const authUsersQueryLimitParser = queryLimitParser.refine((n) => n <= AUTH_USERS_API_GET_LIMIT, {
  message: `limit must be less than ${AUTH_USERS_API_GET_LIMIT}`,
});

export const authUsersQueryPageTokenParser = z.string().refine((n) => n, {
  message: 'page token must be a string',
});

export const makePageTokenParser = <T extends z.ZodType>(parser: T) =>
  z.object({
    pageToken: parser,
  });

export const makePageTokenLimitParser = <LP extends z.ZodType, OP extends z.ZodType>(
  limitParser: LP,
  pageTokenParser: OP
) => makeLimitParser(limitParser).merge(makePageTokenParser(pageTokenParser));

export const queryPageTokenLimitParser = makePageTokenLimitParser(authUsersQueryPageTokenParser, queryLimitParser);

const authUsersListLimitParser = authUsersQueryLimitParser.optional().default(AUTH_USERS_API_GET_LIMIT);
const authUsersListPageTokenParser = authUsersQueryPageTokenParser.optional();
export const authUsersListLimitParamsParser = makeLimitParser(authUsersListLimitParser);
export const authUsersListPageTokenParamsParser = makePageTokenParser(authUsersListPageTokenParser);

// TODO test that limit can't leak over 1000 or to -1
export const authUsersListQueryParser = makePageTokenLimitParser(
  authUsersListLimitParser,
  authUsersListPageTokenParser
);

export const userIdParser = z.string().refine(isUserId);

export const authUsersGetOneParamsParser = z.object({
  id: userIdParser,
});

export const authUsersGetOneResultParser = z.any(/*stub, not schemable*/).transform((a) => a as AuthUser);

export const authUserParser = z.object({
  uid: userIdParser,
  email: z.string().email().optional(),
});

export type AuthUserParsed = z.infer<typeof authUserParser>;
// no extra fields and types match
// eslint-disable-next-line
type AssertAuthUserParser = Subset<
  Omit<auth.UserRecord, 'uid'> & {
    uid: UserId;
  },
  AuthUserParsed
>;

export type AuthUser = AuthUserParsed;

export const authUsersListResultParser = z.any().transform((r) => r as AuthUsersResult);

==> currents-api-types-0.0.40.tgz: package/auth/user/parsers.ts <==
import { z } from 'zod';
import { AUTH_USERS_API_GET_LIMIT } from './constants';
import { isUserId } from './id.utils';
import { AuthUsersResult, UserId } from '../../user/types';
import { auth } from 'firebase-admin';
import { makeLimitParser, queryLimitParser } from '../../common/parsers/queryParams';
import { cast } from '../../utils/types';

// for assertions like one type *looks like* a subtype of another
// eslint-disable-next-line @typescript-eslint/no-unused-vars
export type Subset<T extends U, U> = U;

// for 'get' query params, .or shenanigans
export const authUsersQueryLimitParser = queryLimitParser.refine((n) => n <= AUTH_USERS_API_GET_LIMIT, {
  message: `limit must be less than ${AUTH_USERS_API_GET_LIMIT}`,
});

export const authUsersQueryPageTokenParser = z.string().refine((n) => n, {
  message: 'page token must be a string',
});

export const makePageTokenParser = <T extends z.ZodType>(parser: T) =>
  z.object({
    pageToken: parser,
  });

export const makePageTokenLimitParser = <LP extends z.ZodType, OP extends z.ZodType>(
  limitParser: LP,
  pageTokenParser: OP
) => makeLimitParser(limitParser).merge(makePageTokenParser(pageTokenParser));

export const queryPageTokenLimitParser = makePageTokenLimitParser(authUsersQueryPageTokenParser, queryLimitParser);

const authUsersListLimitParser = authUsersQueryLimitParser.optional().default(AUTH_USERS_API_GET_LIMIT);
const authUsersListPageTokenParser = authUsersQueryPageTokenParser.optional();
export const authUsersListLimitParamsParser = makeLimitParser(authUsersListLimitParser);
export const authUsersListPageTokenParamsParser = makePageTokenParser(authUsersListPageTokenParser);

// TODO test that limit can't leak over 1000 or to -1
export const authUsersListQueryParser = makePageTokenLimitParser(
  authUsersListLimitParser,
  authUsersListPageTokenParser
);

export const userIdParser = z.string().refine(isUserId);

export const authUsersGetOneParamsParser = z.object({
  id: userIdParser,
});

export const authUsersGetOneResultParser = z.any(/*stub, not schemable*/).transform(cast<AuthUser>());

export const authUserParser = z.object({
  uid: userIdParser,
  email: z.string().email().optional(),
});

export type AuthUserParsed = z.infer<typeof authUserParser>;
// no extra fields and types match
// eslint-disable-next-line
type AssertAuthUserParser = Subset<
  Omit<auth.UserRecord, 'uid'> & {
    uid: UserId;
  },
  AuthUserParsed
>;

export type AuthUser = AuthUserParsed;

export const authUsersListResultParser = z.any().transform(cast<AuthUsersResult>());
